/** @file fwUnitTestComponentGenerateResultsReport.ctl

@brief This file contains the functions to write the XML results files.
        A XML file is created for each of the components. 
  
@par Creation Date
		01/04/2014      
        
*/
const int SUCCESS_INDEX = 1; // this array index contains a boolean. True if the test passed.
const int FAILED_MESSAGE_INDEX = 2; // if the test passed this array index will be empty
const int CASE_EXEC_TIME_INDEX = 3; // time given in seconds

const int RESULTS_INDEX = 1;   // the index of the suite results or test case results
const int NUM_TESTS_INDEX = 2; // the index of the number of tests that have been run
const int NUM_FAILED_INDEX = 3;// the index of the number of tests that have failed
const int EXEC_TIME_INDEX = 4; // time given in seconds

// Result directory and files.
const string RESULTS_DIR = PROJ_PATH + "/Results/";
const string RESULTS_INDEX_PATHNAME = RESULTS_DIR + "index.html";  
const string XSL_NAME = "componentResultsFormatting.xsl";
const string XSL_PATHNAME = "/config/" + XSL_NAME; // path within the installationDirectoryPath to the XSL used for formatting the results. This will be copied to the results folder. 

const string LIGHT_RED = "#FF9999";
const string LIGHT_GREEN = "#99FF99";

//----------------------------------------------------------------------------------------------

/** 
  @par Description: Lists the components within the RESULTS_FILE and creates an XML file for each component's results.
  
  @par Usage
	  public

  @param[in] results mapping, key = component name, value = results for a component(@see fwUnitTestComponentGenerateResultsReport_createXmlFileForAComponent())
                                                        
*/
void fwUnitTestComponentGenerateResultsReport_createResultXml(mapping results)
{
  fwUnitTestComponentGenerateResultsReport_setupResultsDir(); 

  // start the results index.html page
  int indexHtmlDocument = xmlNewDocument();
  int htmlElementId = xmlAppendChild(indexHtmlDocument, -1, XML_ELEMENT_NODE, "html");
  // add style to the HTML.
  int styleHtmlElementId = xmlAppendChild(indexHtmlDocument, htmlElementId, XML_ELEMENT_NODE, "style");  
  xmlAppendChild(indexHtmlDocument, styleHtmlElementId, XML_TEXT_NODE, "h1,h2,p,a{	font-family: Arial,sans-serif;	}" + 
                                                                       "p,a{	font-size:12px;	}" + 
                                                                       "span{width:100%; display:inline-block; margin-bottom:5px; margin-bottom:3px; padding-top:2px; padding-left:5px; padding-bottom:3px;}"); 
  
  int hHtmlElementId = xmlAppendChild(indexHtmlDocument, htmlElementId, XML_ELEMENT_NODE, "h1"); 
  xmlAppendChild(indexHtmlDocument, hHtmlElementId, XML_TEXT_NODE, "Unit Testing Results");   
  int pHtmlElementId = xmlAppendChild(indexHtmlDocument, htmlElementId, XML_ELEMENT_NODE, "p");  
  
  for (int i = 1; i <= mappinglen(results); i++)
  {   
    /* add each component to the results index.html. */
    string componentName = mappingGetKey(results, i); 
    
    // add background color
    int spanHtmlElementId = xmlAppendChild(indexHtmlDocument, pHtmlElementId, XML_ELEMENT_NODE, "span");
    string backgroundColor;  
    if (results[componentName][NUM_FAILED_INDEX] > 0) // if any tests fails a red background is added.
    {
      backgroundColor = LIGHT_RED;
    }
    else 
    {
      backgroundColor = LIGHT_GREEN;
    }
    xmlSetElementAttribute(indexHtmlDocument, spanHtmlElementId, "style", "background-color:" + backgroundColor + ";");
    
    // add link to component's XML file    
    int aHtmlElementId = xmlAppendChild(indexHtmlDocument, spanHtmlElementId, XML_ELEMENT_NODE, "a"); 
    xmlSetElementAttribute(indexHtmlDocument, aHtmlElementId, "href", componentName + ".xml");   
    xmlAppendChild(indexHtmlDocument, aHtmlElementId, XML_TEXT_NODE, componentName + " : " + results[componentName][NUM_FAILED_INDEX] + "/" + results[componentName][NUM_TESTS_INDEX] + " tests failed");    
    
    xmlAppendChild(indexHtmlDocument, pHtmlElementId, XML_ELEMENT_NODE, "br");
    
    // create the component's results page ( what the above link links to )
    fwUnitTestComponentGenerateResultsReport_createXmlFileForAComponent(componentName, results[componentName]);   
  }  
  
  // Write out the results index.html page
  xmlDocumentToFile(indexHtmlDocument, RESULTS_INDEX_PATHNAME);
  
  //DebugTN(RESULTS_FILE, " has been written");
}

//----------------------------------------------------------------------------------------------

/** 
  @par Description: Creates the RESULTS_DIR and copies the XSL to the RESULTS_DIR.
  
  @par Usage
	  private
                                                        
*/
private void fwUnitTestComponentGenerateResultsReport_setupResultsDir()
{
  bool ok;
  if (!isdir(RESULTS_DIR)) 
  {
    ok = mkdir(RESULTS_DIR); 
    if ( !ok )
        {
         DebugTN("fwUnitTestComponentGenerateResultsReport.ctl FAILED to create ",RESULTS_DIR);
        }
    
    string xslFullPathName;
    dpGet("fwInstallationInfo.installationDirectoryPath", xslFullPathName);  
    xslFullPathName = xslFullPathName + XSL_PATHNAME; 
    copyFile(xslFullPathName, RESULTS_DIR);
  } 
}

//----------------------------------------------------------------------------------------------

/** 
  @par Description: Creates the XML file for a individual component.
  
  @par Usage
	  private

  @param[in] componentName string
  @param[in] componentResults dyn_anytype, [RESULTS, NUM_TESTS_INDEX, NUM_FAILED_INDEX, EXEC_TIME_INDEX] 
                                           RESULTS is mapping, key = suite name, value = results for a suite(@see fwUnitTestComponentGenerateResultsReport_addTestSuiteToXml()) 
                                                         
*/
private void fwUnitTestComponentGenerateResultsReport_createXmlFileForAComponent(string componentName, dyn_anytype componentResults)
{
  int xmlDocument = xmlNewDocument(); 
  xmlAppendChild(xmlDocument, -1, XML_PROCESSING_INSTRUCTION_NODE, "xml version=\"1.0\"  encoding=\"UTF-8\"");
  xmlAppendChild(xmlDocument, -1, XML_PROCESSING_INSTRUCTION_NODE, "xml-stylesheet type=\"text/xsl\" href=\"" + XSL_NAME + "\"");  
  
  fwUnitTestComponentGenerateResultsReport_addComponentToXml(xmlDocument, componentName, componentResults);   
 
  xmlDocumentToFile(xmlDocument, RESULTS_DIR + "/" + componentName + ".xml");
  //DebugTN(RESULTS_DIR + "/" + componentName + ".xml", " has been written"); 
}

//----------------------------------------------------------------------------------------------

/** 
  @par Description: Adds the results for a component to the xmlDocument.
  
  @par Usage
	  public

  @param[in] xmlDocument int, id for the XML document that is being created. 
  @param[in] componentName string
  @param[in] componentResults dyn_anytype, [RESULTS, NUM_TESTS_INDEX, NUM_FAILED_INDEX, EXEC_TIME_INDEX] 
                                           RESULTS is mapping, key = suite name, value = results for a suite(@see fwUnitTestComponentGenerateResultsReport_addTestSuiteToXml())
                                                         
*/
private void fwUnitTestComponentGenerateResultsReport_addComponentToXml(int xmlDocument, string componentName, dyn_anytype componentResults)
{
  //DebugTN("componentResults = ", componentResults);
  
  int componentElementId = xmlAppendChild(xmlDocument, -1, XML_ELEMENT_NODE, "tests"); 
  xmlSetElementAttribute(xmlDocument, componentElementId , "name", componentName);
  xmlSetElementAttribute(xmlDocument, componentElementId , "tests", componentResults[NUM_TESTS_INDEX]);
  xmlSetElementAttribute(xmlDocument, componentElementId , "failures", componentResults[NUM_FAILED_INDEX]);
  xmlSetElementAttribute(xmlDocument, componentElementId , "time", componentResults[EXEC_TIME_INDEX]);
  
  // add each test suite to the the XML results document.
  mapping componentSuitesResults = componentResults[RESULTS_INDEX];
  for (int i = 1; i <= mappinglen(componentSuitesResults); i++)
  {
    string suiteName = mappingGetKey(componentSuitesResults, i);
    fwUnitTestComponentGenerateResultsReport_addTestSuiteToXml(xmlDocument, componentElementId, suiteName, componentSuitesResults[suiteName], componentName);   
  }
}

//----------------------------------------------------------------------------------------------

/** 
  @par Description: Adds the results for a induvidual test suite to the xmlDocument.
  
  @par Usage
	  public

  @param[in] xmlDocument int, id for the XML document that is being created.
  @param[in] componentElementId int, the id for the component within the xmlDocument
  @param[in] suiteName string 
  @param[in] suitesResults dyn_anytype, [RESULTS, NUM_TESTS_INDEX, NUM_FAILED_INDEX, EXEC_TIME_INDEX] 
                                           RESULTS is mapping, key = test case name, value = results for a case (@see fwUnitTestComponentGenerateResultsReport_addTestCaseToXml())
  @param[in] componentName string                                                        
*/
private void fwUnitTestComponentGenerateResultsReport_addTestSuiteToXml(int xmlDocument, int componentElementId, string suiteName, dyn_anytype suitesResults, string componentName)
{
  int suiteElementId = xmlAppendChild(xmlDocument, componentElementId, XML_ELEMENT_NODE, "testsuite");

  strreplace(suiteName, ".", "_"); // in xUnit a . is used to seperate package names, so jenkins/hudson would think .ctl is the name of the test suite. 
  xmlSetElementAttribute(xmlDocument, suiteElementId , "name", componentName + "." + suiteName);
  xmlSetElementAttribute(xmlDocument, suiteElementId , "tests", suitesResults[NUM_TESTS_INDEX]);
  xmlSetElementAttribute(xmlDocument, suiteElementId , "failures", suitesResults[NUM_FAILED_INDEX]);
  xmlSetElementAttribute(xmlDocument, suiteElementId , "time", suitesResults[EXEC_TIME_INDEX]);
  
  // add each test case to the the XML results document.
  mapping suiteCasesResults = suitesResults[RESULTS_INDEX];
  for (int i = 1; i <= mappinglen(suiteCasesResults); i++)
  {
    string caseName = mappingGetKey(suiteCasesResults, i);
    fwUnitTestComponentGenerateResultsReport_addTestCaseToXml(xmlDocument, suiteElementId, caseName, suiteCasesResults[caseName]);   
  } 
}

//----------------------------------------------------------------------------------------------

/** 
  @par Description: Adds the results for a induvidual test case to the xmlDocument.
  
  @par Usage
	  public

  @param[in] caseResults dyn_anytype, [SUCCESS_INDEX, FAILED_MESSAGE_INDEX, CASE_EXEC_TIME_INDEX ]                                            
                                                         
*/
private void fwUnitTestComponentGenerateResultsReport_addTestCaseToXml(int xmlDocument, int suiteElementId, string caseName, dyn_anytype caseResults)
{
  int caseElementId = xmlAppendChild(xmlDocument, suiteElementId, XML_ELEMENT_NODE, "testcase");
  xmlSetElementAttribute(xmlDocument, caseElementId , "name", caseName);
  xmlSetElementAttribute(xmlDocument, caseElementId , "time", caseResults[CASE_EXEC_TIME_INDEX]);
  
  bool success = caseResults[SUCCESS_INDEX];
  string testCaseMessage = caseResults[FAILED_MESSAGE_INDEX];
  
  if (!success) // write the failed test message to the xml document
  {
    int testfailureXmlElement = xmlAppendChild(xmlDocument, caseElementId, XML_ELEMENT_NODE, "failure");
    xmlSetElementAttribute(xmlDocument, testfailureXmlElement , "message", testCaseMessage);
  }
}

//----------------------------------------------------------------------------------------------
