/** @file fwUnitTestComponentParseConfigurationXml.ctl

@brief Finds which tests to run by parsing the unitTestConfiguration XML file. If the tests are not specified in
        the configuration file then uses fwUnitTestComponentTestDiscovery to discover the tests.

@par Creation Date
		01/04/2014    
  
*/

#uses "FwUnitTestComponent/fwUnitTestComponentTestDiscovery.ctl"

/** 
  @par Description: Gets the tests to run from the unitTestConfiguration XML file.
  
  @par Usage
	  Public

  @param[in]  configPathName string
  @param[out] testsToRun mapping, key= component name, value=(mapping key = suitesToRun, value = test cases to run)
  
*/
void fwUnitTestComponentParseConfigurationXml_getTestsToRun(string configPathName, mapping &testsToRun)
{
  int errLine, errColumn;
  string errMsg;  
  
  int configXml = xmlDocumentFromFile(configPathName, errMsg, errLine, errColumn); 
  if(configXml == -1)
  {
    DebugTN("Bad file conversion to XML: " + configPathName);
    DebugTN("Test discovery will be used. ");
  }
  else  
  { 
    int node = xmlFirstChild(configXml);
    fwUnitTestComponentParseConfigurationXml_recurseXmlConfig(configXml, node, testsToRun);
  }
  
  // bad file conversion or no tests given in  XML, so run all tests
  if (mappinglen(testsToRun) <= 0) 
  {
    fwUnitTestComponentTestDiscovery_getTests(testsToRun);
  }
    
  xmlCloseDocument(configXml);
  //DebugTN(testsToRun); 
}

//------------------------------------------------------------------------------------------

/** 
  @par Description: Finds the <component> sections of the XML file  
  @par Usage
	  Public

  @param[in]  configXml int, The unitTestConfiguration document ID.
  @param[in]  node int, The ID of the current XML node. 
  @param[out] testsToRun mapping, key= component name, value=(mapping key = suitesToRun, value = test cases to run)    
*/
private void fwUnitTestComponentParseConfigurationXml_recurseXmlConfig(int configXmlFile, int node, mapping &testsToRun)
{ 
  dyn_int children;
  string nodeName = xmlNodeName(configXmlFile, node); 
  
  // finds and parses each component section
  if(nodeName == "component") 
  {       
    fwUnitTestComponentParseConfigurationXml_recurseComponentBlock(configXmlFile, node, testsToRun); 
  }
  else 
  {
    xmlChildNodes(configXmlFile, node, children);
    for(int i=1; i <= dynlen(children); i++)
    {          
      fwUnitTestComponentParseConfigurationXml_recurseXmlConfig(configXmlFile, children[i], testsToRun);
    }
  }
}

//------------------------------------------------------------------------------------------

/** 
  @par Description: Gets the component name from the XML file.
                    Gets the test suites from parsing the XML file or if no test suite is specified 
                      gets the tests using test discovery.
  
  @par Usage
	  Public

  @param[in]  configXml int, The unitTestConfiguration document ID. 
  @param[in]  node int, The ID of the current XML node. 
  @param[out] testsToRun mapping, key= component name, value=(mapping key = suitesToRun, value = test cases to run)   
*/
private void fwUnitTestComponentParseConfigurationXml_recurseComponentBlock(int configXmlFile, int node, mapping &testsToRun)
{
  mapping testSuites;  
  string componentName;
  dyn_int children;
  
  int parent = xmlParentNode(configXmlFile, node);
  string parentName = xmlNodeName(configXmlFile, parent); 
    
  if(parentName == "component") //we need the parent tag to be <component> as pvss thinks of the text is within <#text> tags
  {   
    // get the component name
    componentName = xmlNodeValue(configXmlFile, node); 
    strreplace(componentName, ".xml", "");
    componentName = strrtrim(baseName(componentName), "\n \r \t");  
    //componentName = strrtrim(baseName(componentName), ".xml \n \r \t");  
    componentName = strltrim(componentName, " \n \r \t");  
    //DebugTN("component name = ", componentName );
    
    // check if <testSuite> tag has been given
    xmlChildNodes(configXmlFile, parent, children);
    if (dynlen(children) > 1) // >1 used as <#text> is 1 <testSuite> would then be a 2nd
    {  
      //DebugTN("suite parse" );
      fwUnitTestComponentParseConfigurationXml_recurseSuiteBlock(configXmlFile, parent, testSuites, componentName);           
    }
    else // no test suite given, so use test discovery
    { 
      //DebugTN("suite discovery" );
      fwUnitTestComponentTestDiscovery_testDiscoveryForComponent(componentName, testSuites);
    }
    
   testsToRun[componentName] = testSuites;
   mappingClear(testSuites);
  }
  else 
  {
    xmlChildNodes(configXmlFile, node, children);    
    for(int i=1; i <= dynlen(children); i++)
    { 
      if (xmlNodeName(configXmlFile, children[i]) != "testSuite")// looking for <#text> not <testSuite> 
      {
        fwUnitTestComponentParseConfigurationXml_recurseComponentBlock(configXmlFile, children[i], testsToRun);
      }
    }
  }  
} 

//------------------------------------------------------------------------------------------

/** 
  @par Description: Gets the suite name(and path).
                    Gets the test cases from parsing the XML file or if no test case is specified 
                      gets the tests using test discovery.
  
  @par Usage
	  Public

  @param[in]  configXml int, The unitTestConfiguration document ID. 
  @param[in]  node int, The ID of the current XML node. 
  @param[out] testsToRun mapping, key= component name, value=(mapping key = suitesToRun, value = test cases to run)   
    
*/
private void fwUnitTestComponentParseConfigurationXml_recurseSuiteBlock(int configXmlFile, int node, mapping &testSuites, string componentName)
{
  dyn_int children; 
  int parent = xmlParentNode(configXmlFile, node);
  string parentName = xmlNodeName(configXmlFile, parent);  
  string nodeName = xmlNodeName(configXmlFile, node);
   
  if(parentName == "testSuite") //we need the parent tag to be <component> as pvss thinks of the text is within <#text> tags.....but not when you use xmlNodeValue :/ 
  {  
    // gets the test suite name. If the suite name has no .ctl then appends .ctl to the name. 
    string testSuiteName = xmlNodeValue(configXmlFile, node);  
    testSuiteName = strrtrim(testSuiteName, "\n\r\t ");
    testSuiteName = strltrim(testSuiteName, " \n \r \t"); 
    if (strpos(testSuiteName, ".ctl") < 0) 
    {
      testSuiteName = testSuiteName + ".ctl";  
    }       
    testSuiteName = fwUnitTestComponentTestDiscovery_getTestScriptPathName(componentName, testSuiteName);
    if (testSuiteName == "") // could not find the given testSuiteName
    {
      DebugTN("Could not find ", xmlNodeValue(configXmlFile, node));
      return;
    }
    //DebugTN("testSuiteParser name = ", testSuiteName ); 
  
    // get the test cases
    xmlChildNodes(configXmlFile, parent, children);
    if (dynlen(children) > 1) // get test cases from the XML file
    {
     dyn_string testCases;
     fwUnitTestComponentParseConfigurationXml_recurseTestCaseBlock(configXmlFile, parent, testCases);
     testSuites[testSuiteName] = testCases;
     //DebugTN("case parse" );
    }
    else // get test case using test discovery
    {
      //DebugTN("case discovery" );
      dyn_string testRoutines;
      fwUnitTestComponentTestDiscovery_getTestRoutinesFromScript(testSuiteName, testRoutines);
      testSuites[testSuiteName] =  testRoutines; 
    }
  }
  else 
  {
    xmlChildNodes(configXmlFile, node, children);
    for(int i=1; i <= dynlen(children); i++)
    {
      if (xmlNodeName(configXmlFile, children[i]) != "testCase")// looking for <#text>, teastCase needs to be ignored.
      {
        fwUnitTestComponentParseConfigurationXml_recurseSuiteBlock(configXmlFile, children[i], testSuites, componentName);
      }
    }
  }
 // DebugTN("test suites = ", testSuites);
} 

//------------------------------------------------------------------------------------------

/** 
  @par Description: Gets the list of test cases from parsing the XML file.
  
  @par Usage
	  Public

  @param[in]  configXml int, The unitTestConfiguration document ID. 
  @param[in]  node int, The ID of the current XML node. 
  @param[out] testsToRun mapping, key= component name, value=(mapping key = suitesToRun, value = test cases to run)   

*/
private void fwUnitTestComponentParseConfigurationXml_recurseTestCaseBlock(int configXmlFile, int node, dyn_string &testCases)
{
  dyn_int children, base; 
  int parent = xmlParentNode(configXmlFile, node);
  string name = xmlNodeName(configXmlFile, parent);  
  string nodeName = xmlNodeName(configXmlFile, node);
 
  if(name == "testCase") //we need the parent tag to be <component> as pvss thinks of the text is within <#text> tags.....but not when you use xmlNodeValue :/ 
  {   
    string testCase = xmlNodeValue(configXmlFile, node);
    //DebugTN("testcase name = ", testCase );   
    testCase = strrtrim(testCase, "\n\r\t ");
    testCase = strltrim(testCase, " \n \r \t"); 
    // add () to the test case routine 
    if (testCase[strlen(testCase)-1] != ')') 
    {
      testCase = testCase + "()";
    } 
    dynAppend(testCases, testCase);
  }
  else 
  {
    xmlChildNodes(configXmlFile, node, children);  
    for(int i=1; i <= dynlen(children); i++)
    {
      fwUnitTestComponentParseConfigurationXml_recurseTestCaseBlock(configXmlFile, children[i], testCases);
    }
  } 
 // DebugTN("testcases = ", testCases );
}

