/** @file fwUnitTestComponentScriptToEval
 
@brief fwUnitTestComponentTestRunner.ctl will read this script in as a string, append the test suite script 
        and pass the string to evalScript. 
       This script calls the setup/teardown routines and each of the test cases which have been passed as arguments to it.

@par Creation Date
		30/05/2014      
  
*/

/**
  
  @par Usage
	  fwUnitTestComponentTestRunner.ctl 

  @param[in] testCases dyn_string
  @param[in] suiteSetup string
  @param[in] suiteTeardown string
  @param[in] caseSetup string
  @param[in] caseTeardown string
  
  The following parameters are only given when run using the panel:
  @param[in] testSelectionTree shape 
  @param[in] testProgressBar shape  
  @param[in] feedbackText shape 
  @param[in] feedbackProgressText shape, The text field found to the right of the testProgressBar.  
  @param[in] numTestsToRun int, Used for updating the testProgressBar and feedbackProgressText as the tests are completed.
    
  @param[in] totalTestsPassed int, Will also be returned within the dyn_anytype
  @param[in] totalTestsPerformed int, Will also be returned within the dyn_anytype
  
  @return dyn_anytype [suiteResults, totalTestsPerformed, totalTestsPassed, numberOfTestCasesPerformed, testCasesPassed]
*/

#uses "FwUnitTestComponent/fwUnitTestComponentUpdateGui.ctl"

dyn_anytype main(dyn_string testCases, string suiteSetup, string suiteTeardown, string caseSetup, string caseTeardown, shape testSelectionTree, shape testProgressBar, shape feedbackText, shape feedbackProgressText, int numTestsToRun, int totalTestsPassed, int totalTestsPerformed)
{
  bool success = true;
  string failedMessage = ""; 
  string strExecTime = "0";
  string testCase;  
  int testCasesPassed = 0, numberOfTestCasesPerformed = 0;   
  mapping suiteResults; // key = test case name, value = [success, failedMessage, strExecTime]
  //DebugTN("testCases = ", testCases ); 
 
  // suiteSetup
  runSetupOrTeardownRoutine(suiteSetup);  
  
  for (int testCaseIndex = 1; testCaseIndex <= dynlen(testCases); testCaseIndex++)
  {  
    testCase = testCases[testCaseIndex];
    
    // caseSetup
    runSetupOrTeardownRoutine(caseSetup);
        
    success = false;
    failedMessage = "";
    int evalResult; 
    time start = getCurrentTime(); 
   
    try
    {
      // run the test case
      dyn_anytype results; 
      results[1] = true; 
      results[2] = "";
     
      try 
      {  
         string testCaseName = testCase;
         strreplace(testCaseName, "()", "");
         callFunction(testCaseName);
      }
      catch 
      { 
         results[1] = false; 
         results[2] = getErrorText(getLastException());       
         DebugN("Error in " + testCase );
      }   
       
      // check test result
      if (!results[1])
      {
        DebugN("Failure:", testCase, results[2]);
        failedMessage = "Failure:" + testCase + " " + results[2];
      }
      success = results[1];
    }
    catch // error in test script
    {
      success = false;
      failedMessage = getLastException();
    }
    
    time execTime = getCurrentTime() - start;
    strExecTime = formatExecutionTime(execTime);
    
    // add test case results to suiteResults mapping    
    numberOfTestCasesPerformed = numberOfTestCasesPerformed + 1;
    totalTestsPerformed = totalTestsPerformed + 1;
    if (success)
    {
      testCasesPassed = testCasesPassed + 1;
      totalTestsPassed = totalTestsPassed + 1;
      failedMessage = "";
    }
    suiteResults[testCase] = makeDynAnytype(success, failedMessage, strExecTime);
      
    // display the results for test case 
    fwUnitTestComponentUpdateGui_updateGuiTree(testCase, success, failedMessage, strExecTime, testSelectionTree);
    fwUnitTestComponentUpdateGui_displayProgress(totalTestsPassed, totalTestsPerformed, testProgressBar, feedbackText, feedbackProgressText, numTestsToRun);

    
    // caseTeardown
    runSetupOrTeardownRoutine(caseTeardown);
  }
  
  // suiteTeardown
  runSetupOrTeardownRoutine(suiteTeardown);
  
  return makeDynAnytype(suiteResults, totalTestsPerformed, totalTestsPassed, numberOfTestCasesPerformed, testCasesPassed);  
}

//----------------------------------------------------------------------------------------------
private string formatExecutionTime(time t) {
  int iLen = strlen((string)second(t)) + 4;
  float ftime = (float) t;
  return strformat("\\left{%8.3}", iLen, ftime);
}
//----------------------------------------------------------------------------------------------

/** 
  @par Description: Runs the given setup/teardown routine.
  
  @par Usage
	  public

  @param[in] routine string                                                        
*/
private void runSetupOrTeardownRoutine(string routine)
{
  // suiteTeardown
  try
  {
    if (routine != "")
    {
      strreplace(routine, "()", "");
      callFunction(routine);
    }
  }
  catch
  {
    DebugTN("Failed to run setup/teardown rotuien = " + routine);//TODO : Display this in user friendly way. (eg. In panel)
    DebugTN(getLastException());
  }
}
