/** @file fwUnitTestComponentTestRunner.ctl
 
@brief This file is used by the fwUnitTestComponentTestRunner.pnl and fwUnitTestComponentStartTestRunner.ctl(when using the python harness)
        to run the unit tests. 

@par Creation Date
		01/04/2014      
  
*/
#uses "FwUnitTestComponent/fwUnitTestComponentGenerateResultsReport.ctl"
#uses "FwUnitTestComponent/fwUnitTestComponentUpdateGui.ctl"
#uses "FwUnitTestComponent/fwUnitTestComponentTestDiscovery.ctl"

//------------------------------------------------------------------------------------------

const string INSTALLATION_DIRECTORY_DPE = "fwInstallationInfo.installationDirectoryPath";


int totalTestsPassed = 0;
int totalTestsPerformed = 0;

//------------------------------------------------------------------------------------------

/** 
  @par Description: Runs all of the given tests.
  
  @par Usage
	  Public

  @param[in] testsToPerform mapping, (key = component name, value = mapping(key = suite name, value = [case name, ]))
  
  The following parameters are only given when run using the panel:
  @param[in] testSelectionTree shape 
  @param[in] testProgressBar shape  
  @param[in] feedbackText shape 
  @param[in] feedbackProgressText shape, The text field found to the right of the testProgressBar.  
  @param[in] numTestsToRun int, Used for updating the testProgressBar and feedbackProgressText as the tests are completed.
*/
void fwUnitTestComponentTestRunner_runTests(mapping testsToPerform, shape testSelectionTree = 0, shape testProgressBar = 0, shape feedbackText = 0, shape feedbackProgressText = 0, int numTestsToRun = 0)
{
  dyn_string componentNames = mappingKeys(testsToPerform);
  totalTestsPassed = 0;
  totalTestsPerformed = 0;
  
  mapping results;// key = component name, value = [results for a component, NUM_TESTS_INDEX, NUM_FAILED_INDEX, EXEC_TIME_INDEX]     // consts set in fwUnitTestComponentGenerateResultsReport.ctl
                    //where "results for a component" is mapping(key = suite name, value = [results for a suite, NUM_TESTS_INDEX, NUM_FAILED_INDEX, EXEC_TIME_INDEX])
                    //where "results for a suite" is mapping(key = case name, value = [SUCCESS_INDEX, FAILED_MESSAGE_INDEX, CASE_EXEC_TIME_INDEX])
  // run the tests for each of the components 
  for (int i = 1; i <= dynlen(componentNames); i++)
  {
    string componentName = componentNames[i];
    mapping componentTestSuites = testsToPerform[componentName];
    fwUnitTestComponentTestRunner_runTestsForComponent(componentName, componentTestSuites, testSelectionTree, testProgressBar, feedbackText, feedbackProgressText, numTestsToRun, results);   
  }  

  fwUnitTestComponentGenerateResultsReport_createResultXml(results); 
}

//------------------------------------------------------------------------------------------

/** 
  @par Description: For each of the tests suites runs the suiteSetup(), the tests cases(including the setup() and teardown()) and the suiteTeardown().
  
  @par Usage
	  private

  @param[in] componentName string 
  @param[in] componentTestSuites mapping, key = script, value = routine names
  @param[in] testSelectionTree shape, 
  @param[in] testProgressBar shape, 
  @param[out] results mapping, key = component name, value = results for a component @see fwUnitTestComponentGenerateResultsReport.ctl
*/
private void fwUnitTestComponentTestRunner_runTestsForComponent(string componentName, mapping componentTestSuites, shape testSelectionTree, shape testProgressBar, shape feedbackText, shape feedbackProgressText, int numTestsToRun, mapping &results)
{
  // keeps all the results for a component
  mapping componentResults;  // key = suite name, value = [suiteResults, NUM_TESTS, NUM_FAILED, EXEC_TIME]
  int componentNumberOfTestsPerformed = 0, componentTestsPassed = 0;  
  time componentExecTime = 0;
  time componentStartTime = getCurrentTime();  
  
  // stores results for a test suite 
  mapping suiteResults; // key = test case name, value = [SUCCESS, FAILED_MESSAGE, EXEC_TIME]
  time suiteExecTime = 0;   
  int suiteNumberOfTestsPerformed, suiteTestsPassed;  
  
  string evalScriptString;
  fileToString(getScriptToEvalPathname(), evalScriptString); 
                    
  DebugTN("*** *** *** fwUnitTestComponentTestRunner_runTestsForComponent is about to eval for componentName = ",componentName);
  
  // run each test suite
  for (int i = 1; i <= mappinglen(componentTestSuites); i++)
  {
    // get the content of the suite test script
    string testSuiteScriptPathName = mappingGetKey(componentTestSuites, i);
    string testSuiteScriptContent;
    fileToString(testSuiteScriptPathName, testSuiteScriptContent);  
    
    mappingClear(suiteResults);
    dyn_string testCases = componentTestSuites[testSuiteScriptPathName];      
    time suiteStartTime = getCurrentTime(); // used to work out how long executing all the tests in the suite script takes
          
    // get the setup and teardown routines
    string caseSetupRoutine; // if no setup/teardown routine is found we set the routine to "".
    if ( !fwUnitTestComponentTestDiscovery_getCaseSetupRoutineFromScript(testSuiteScriptPathName, caseSetupRoutine))  caseSetupRoutine = "";  
    string suiteSetupRoutine;
    if ( !fwUnitTestComponentTestDiscovery_getSuiteSetupRoutineFromScript(testSuiteScriptPathName, suiteSetupRoutine))  suiteSetupRoutine = "";  
    string caseTeardownRoutine;
    if ( !fwUnitTestComponentTestDiscovery_getCaseTeardownRoutineFromScript(testSuiteScriptPathName, caseTeardownRoutine))  caseTeardownRoutine = "";  
    string suiteTeardownRoutine;
    if ( !fwUnitTestComponentTestDiscovery_getSuiteTeardownRoutineFromScript(testSuiteScriptPathName, suiteTeardownRoutine) ) suiteTeardownRoutine = "";   
    
    // run the tests in the testSuiteScript
    dyn_anytype evalScriptResults; // temporary store for the results. To allow multiple values to be returned from the evalScript. 
    string scriptToEval = evalScriptString + "  " + testSuiteScriptContent;
    
    DebugTN("*** *** *** fwUnitTestComponentTestRunner_runTestsForComponent is about to eval for suite = ",testSuiteScriptPathName);
//    DebugTN("<<< <<<");
//    DebugTN(scriptToEval);
//    DebugTN(">>> >>>");
  
    string sImportedFilesContent = getImportedFilesContent(testSuiteScriptContent);
    
    scriptToEval = evalScriptString + "  " + sImportedFilesContent + "   " + testSuiteScriptContent;
    if (!checkScript(scriptToEval)) {      
      totalTestsPerformed += dynlen(testCases);
      componentNumberOfTestsPerformed++;
      fwUnitTestComponentTestRunner_updateTestSuiteResults(componentResults, makeDynAnytype(0, "0", "0.0"), testSuiteScriptPathName, suiteStartTime, 1, 0, testSelectionTree, "Syntax error");
      fwUnitTestComponentUpdateGui_displayProgress(totalTestsPassed, totalTestsPerformed, testProgressBar, feedbackText, feedbackProgressText, numTestsToRun);
      continue;
    }
    
    try{
      evalScript(evalScriptResults, 
                 scriptToEval, // script
                 makeDynString(), // dollar parameters
                 testCases, // args to the main()
                 suiteSetupRoutine, 
                 suiteTeardownRoutine, 
                 caseSetupRoutine, 
                 caseTeardownRoutine, 
                 testSelectionTree, 
                 testProgressBar, 
                 feedbackText, 
                 feedbackProgressText, 
                 numTestsToRun, 
                 totalTestsPassed, 
                 totalTestsPerformed);
    
      suiteResults = evalScriptResults[1];
      totalTestsPerformed = evalScriptResults[2];
      totalTestsPassed = evalScriptResults[3];  
      suiteNumberOfTestsPerformed = evalScriptResults[4];
      suiteTestsPassed = evalScriptResults[5];
    }
    catch // error when running the evalScript(). So far this has always been due to syntax errors in the test suite script.
    {
      componentNumberOfTestsPerformed += dynlen(testCases);
      fwUnitTestComponentTestRunner_updateTestSuiteResults(componentResults, makeDynAnytype(0, "0", "0.0"), testSuiteScriptPathName, suiteStartTime, 1, 0, testSelectionTree, "Syntax error");
      fwUnitTestComponentUpdateGui_displayProgress(totalTestsPassed, totalTestsPerformed, testProgressBar, feedbackText, feedbackProgressText, numTestsToRun);
      continue;
    }
   
    componentNumberOfTestsPerformed = componentNumberOfTestsPerformed + suiteNumberOfTestsPerformed;
    componentTestsPassed = componentTestsPassed + suiteTestsPassed;  
    fwUnitTestComponentTestRunner_updateTestSuiteResults(componentResults, suiteResults, testSuiteScriptPathName, suiteStartTime, suiteNumberOfTestsPerformed, suiteTestsPassed, testSelectionTree);    
  } 

  // display the results for the component  
  componentExecTime = getCurrentTime() - componentStartTime;
  string strComponentExecTime = formatExecutionTime(componentExecTime);
  fwUnitTestComponentUpdateGui_updateGuiTree(componentName, (componentNumberOfTestsPerformed == componentTestsPassed), "", strComponentExecTime, testSelectionTree);
  
  results[componentName] = makeDynAnytype(componentResults, componentNumberOfTestsPerformed, (componentNumberOfTestsPerformed - componentTestsPassed), strComponentExecTime);
  //DebugTN("results = ", results);
}
//----------------------------------------------------------------------------------------------
private string formatExecutionTime(time t) {
  int iLen = strlen((string)second(t)) + 4;
  float ftime = (float) t;
  return strformat("\\left{%8.3}", iLen, ftime);
}
//------------------------------------------------------------------------------------------

private void fwUnitTestComponentTestRunner_updateTestSuiteResults(mapping &componentResults, mapping suiteResults, string testSuiteScriptPathName, time suiteStartTime, 
                                                                  int suiteNumberOfTestsPerformed, int suiteTestsPassed, shape testSelectionTree, string failedMessage="") {
  bool success = suiteNumberOfTestsPerformed - suiteTestsPassed == 0;
  time suiteExecTime = getCurrentTime() - suiteStartTime;
  string strSuiteExecTime = formatExecutionTime(suiteExecTime);
  fwUnitTestComponentUpdateGui_updateGuiTree(testSuiteScriptPathName, success, failedMessage, strSuiteExecTime, testSelectionTree);
  componentResults[testSuiteScriptPathName] = makeDynAnytype(suiteResults, suiteNumberOfTestsPerformed, success, strSuiteExecTime);
}

//------------------------------------------------------------------------------------------
 
/** 
  @par Description: Finds the script to be eval'ed. Could be in PROJ_PATH or in the installation directory.
  
  @par Usage
	  Public

  @return string scriptToEvalPathname

*/
private string getScriptToEvalPathname()
{
  string path = getPath(LIBS_REL_PATH, "FwUnitTestComponent/fwUnitTestComponentScriptToEval.ctl");
  if (path == "") {
    throw(makeError("", PRIO_SEVERE, ERR_IMPL, 86, "Cannot find fwUnitTestComponentScriptToEval.ctl"));
  }
  return path;
}

//------------------------------------------------------------------------------------------
/**
 * Get the list of imported files in a script (searching for the '#uses' statements
 * @param script The script contents
 */
private dyn_string getImportedFiles(string script) {
  dyn_string dsLines = strsplit(script, '\n');
  dyn_string dsImported;
  
  for (int i=1; i<=dynlen(dsLines); i++) {
    string sLine = strltrim(dsLines[i]);
    if (strpos(sLine, "#uses") == 0) {
      int iFrom = strpos(sLine, "\"");
      if (iFrom < 0) {
        continue;
      }
      int iTo = strpos(sLine, "\"", iFrom+1);
      if (iTo < 0) {
        continue;
      }
      
      string sFile = substr(sLine, iFrom+1, iTo-iFrom-1);
      dynAppend(dsImported, sFile);
    }
  }
  return dsImported;
}

//------------------------------------------------------------------------------------------
/**
 * Gets the contents of the imported files in a script.
 * @param sScript The script contents
 */
private string getImportedFilesContent(string sScript) {
  string sContent;
  
  dyn_string dsImported = getImportedFiles(sScript);
  for (int i=1; i<=dynlen(dsImported); i++) {
    string sPath = getPath(LIBS_REL_PATH, dsImported[i]);
    if (sPath == ""){
      sPath = getPath(SCRIPTS_REL_PATH, dsImported[i]);
    }
    if (sPath != "") {
      string sFileContent;
      fileToString(sPath, sFileContent);
      sContent += sFileContent + "\n\n\n";
    } 
  }
  return sContent;
}
//------------------------------------------------------------------------------------------
